local addonInfo, privateVars = ...

if nkAM_Level ~= nil then return end

nkAM_Level = {}

privateVars.modules = {}
privateVars.gridFinished = false

privateVars.internal = {}

privateVars.uiElements = {}
privateVars.uiElements.context = UI.CreateContext("nkAM_Level")

function privateVars.internal.main(addon)

	if (addon == addonInfo.toc.Identifier) then
		if nkAAPI.checkVersion (addonInfo.toc.Identifier, "nkAdvisor", "4.1.2") == false then return end		

		nkAAPI.addModule(addonInfo.toc.Identifier, addonInfo.toc.Version, 1, privateVars.langTexts.tabLootTables, privateVars.internal.buildPanel) 
		nkAAPI.registerModuleFunc('nkAdvisor_Level', 'getDropLocation', privateVars.internal.getDropLocation) 
	
		privateVars.internal.loadItems()	
		
		EnKai.version.init(addonInfo.toc.Identifier, addonInfo.toc.Version)
	end
	
end

function privateVars.internal.settings() 
	
	local settings = { showPercent = false  }
	
	if nkAM_Level_Setup == nil then
		nkAM_Level_Setup = settings
	else
		for k, v in pairs (settings) do if nkAM_Level_Setup[k] == nil then nkAM_Level_Setup[k] = v end end
	end	
	
end

table.insert(Event.Addon.Load.End, {privateVars.internal.main, "nkAdvisor_Level", "addonLoaded"})
table.insert(Event.Addon.SavedVariables.Load.End, {privateVars.internal.settings, "nkAdvisor_Level", "vaiablesLoaded"})

function privateVars.internal.loadItems ()

	local slots = {}
	privateVars.dbCategories = {}

	for slot, slotData in pairs(privateVars.db) do
	  table.insert(privateVars.dbCategories, slot)
			
		for subSlot, subSlotData in pairs (slotData) do
			if subSlot ~= "itemType" then    
        table.insert(slots, {slot, subSlot})
      end
		end
	end

	local loadItemCoRoutine = coroutine.create(
		function ()
			for idx = 1, #slots, 1 do
				local slot = slots[idx][1]
				local subslot = slots[idx][2]
				
				local tempData = nkAAPI.unzippData (privateVars.db[slot][subslot])
				
				for key, details in pairs (tempData) do
					nkItemBase.Execute.addItem (slot, subslot, key, details)
				end					
				coroutine.yield(idx)
			end
		end)

	EnKai.coroutines.add ( { func = loadItemCoRoutine, counter = #slots, active = true } )

end

function privateVars.internal.buildPanel(frame, uiObject)

	if Inspect.System.Secure() == false then Command.System.Watchdog.Quiet() end
	
	privateVars.internal.buildLootTableSelection(frame, uiObject)
	
	privateVars.uiElements.lootTable = privateVars.internal.lootTable('', '', frame.tabPane:GetBodyFrame())
	
end

function privateVars.internal.switchToLootTable (loottype, key)

  if privateVars.gridFinished == false then return end
	
	privateVars.uiElements.lootTable:show(true)
	privateVars.uiElements.lootTable:changeTable(loottype, key)
	
end

function privateVars.internal.buildLootTableSelection (frame)

  frame.tabPane = EnKai.uiCreateFrame("nkTabpaneMetro", frame:GetName() .. ".tabPane", frame)
  frame.tabPane:SetPoint("TOPLEFT", frame, "TOPLEFT")
  frame.tabPane:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, -50)
  frame.tabPane:SetLayer(1)
  frame.tabPane:SetBorder(false)
  frame.tabPane:SetVertical(true)
  frame.tabPane:SetColor(nil, { type = 'solid', r = 0.055, g = 0.133, b = 0.208, a = 1}, nil, nil)

	local panes = {}
	
	local loot = {
		{"chronicle",privateVars.langTexts.lootTableSel.headerChronicle},
		{"currency",privateVars.langTexts.lootTableSel.headerCurrency},
		{"dungeon",privateVars.langTexts.lootTableSel.headerDungeon},
		{"expert",privateVars.langTexts.lootTableSel.headerExpert},
		{"faction",privateVars.langTexts.lootTableSel.headerFaction},
		{"master",privateVars.langTexts.lootTableSel.headerMaster},
		{"planeware",privateVars.langTexts.lootTableSel.headerPlaneWare},
		{"raid10",privateVars.langTexts.lootTableSel.headerRaid10},
		{"raid20",privateVars.langTexts.lootTableSel.headerRaid20}
	}
	
	for idx = 1, #loot, 1 do
	  local flag = false
	  if idx == #loot then flag = true end
		local pane = privateVars.internal.lootCategory(frame, loot[idx][1])
		frame.tabPane:AddPane( { label = loot[idx][2], frame = pane, initFunc = function () pane:activate() end}, flag)
	end
	
	--frame.tabPane:UpdatePanes()
	
end

function privateVars.internal.getDropLocation (data)
  
	local lootType, sourceKey = privateVars.internal.getDropInfo(data.key)
	local bossInfo = ''
	local instanceInfo = ''

	if lootType == nil or sourceKey == nil then
		instanceInfo = privateVars.langTexts.error.srcnotfound
	elseif lootType == 'profession' then
		instanceInfo = privateVars.langTexts.profession[sourceKey]
		local augmentItem = nkItemBase.Query.getAugmentationByKey(data.augmentKey)

		if augmentItem ~= nil then
			local augmentDesc = nil
			augmentDesc = augmentItem[string.lower(EnKai.tools.lang.getLanguageShort())]
			bossInfo = string.format (privateVars.langTexts.msgCraftingAugmentedBy, augmentDesc)
		end
	elseif lootType == 'special' then
		return '', sourceKey
	else
		for k, v in pairs (privateVars.loot[lootType].source) do
			if k == sourceKey then
				if privateVars.loot[lootType].locations[k] == nil then
					if privateVars.loot[lootType].locations[nkAAPI.getFaction()][k] ~= nil then
						instanceInfo = privateVars.loot[lootType].locations[nkAAPI.getFaction()][k]
					else
						bossInfo = ''
						instanceInfo = privateVars.langTexts.error.srcnotfound
					end
				else
					bossInfo = w 
					instanceInfo = privateVars.loot[lootType].locations[k]
				end
				break
			else 
				for l, w in pairs (v) do
					if l == sourceKey then
						if privateVars.loot[lootType].locations[k] == nil then
							if privateVars.loot[lootType].locations[nkAAPI.getFaction()][k] ~= nil then
								bossInfo = ''
								if privateVars.langTexts.lootTableText[lootType] ~= nil then bossInfo = privateVars.langTexts.lootTableText[lootType] end
								instanceInfo = privateVars.loot[lootType].locations[nkAAPI.getFaction()][k]
							else
								bossInfo = ''
								instanceInfo = privateVars.langTexts.error.srcnotfound
							end
						elseif lootType == 'event' then 
							bossInfo = privateVars.langTexts.txtEvent 
							instanceInfo = privateVars.loot[lootType].locations[k]
						else
							bossInfo = w
							instanceInfo = privateVars.loot[lootType].locations[k]
						end
						break
					end
				end
			end
		end
	end
	
	if bossInfo == '' and instanceInfo == '' then instanceInfo = privateVars.langTexts.error.srcnotfound end
	
	return bossInfo, instanceInfo

end

function privateVars.internal.getDropInfo (key)

	for loottype, v in pairs (privateVars.loot) do
		if v.items ~= nil then
			for sourceKey, drops in pairs (v.items) do
				if type(drops) == 'table' then
					local pos = EnKai.tools.table.getTablePos(drops, key)
					if pos ~= -1 then return loottype, sourceKey end
				end
			end
		end
		
		if v.other ~= nil then
			for sourceKey, drops in pairs (v.other) do
				if type(drops) == 'table' then
					local pos = EnKai.tools.table.getTablePos(drops, key)
					if pos ~= -1 then return loottype, sourceKey end
				end
			end
		end
		
		if v.rune ~= nil then
			for sourceKey, drops in pairs (v.rune) do
				if type(drops) == 'table' then
					local pos = EnKai.tools.table.getTablePos(drops, key)
					if pos ~= -1 then return loottype, sourceKey end
				end
			end
		end
	end
	
	local profession = nkItemBase.Query.professionOfItem(key)
	if profession ~= nil then return 'profession', profession end
	
	local itemData = nkItemBase.getByKey(key)
	if string.find (itemData.de, 'Antiquierte') == 1 then return 'special', privateVars.langTexts.txtDropsNoLonger end		
	
	return nil
	
end